#pragma once

/*
* Simple Serial Protocol
*
* This product includes software developed by Nick Leeman.
* Simple Serial Protocol Project (https://git.aterve.com/Frozenverse/SimpleSerialProtocol).
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
*/

#include <Arduino.h>

#include "Packet.h"
#include "SerialConnection.h"

class SimpleSerialProtocol
{
    private:
        SerialConnection serialConnection;
    public:
        bool init(HardwareSerial &serialHandle, unsigned long baud);
        bool receivePackets(void (*callback)(Packet));
        bool sendPacket(char* ID, char* command, char* data);
};

bool SimpleSerialProtocol::init(HardwareSerial &serialHandle, unsigned long baud) {
    serialConnection.init(serialHandle, baud);
    return true;
};

bool SimpleSerialProtocol::receivePackets(void (*callback)(Packet)) {
    // Try to get buffer from serial device
    char* receivedBuffer = serialConnection.handle();

    // Check if received a complete buffer
    if (receivedBuffer != NULL) {
        // Buffer pointer
        char* strtokIndex;

        // Marker
        char marker[1] = "";
        marker[0] = serialConnection.packetMidMarker;

        // Get ID
        strtokIndex = strtok(receivedBuffer, marker);
        char id[strlen(strtokIndex)];
        strcpy(id, strtokIndex);

        // Get Type
        strtokIndex = strtok(NULL, marker);
        char type[strlen(strtokIndex)];
        strcpy(type, strtokIndex);
        
        // Get Data
        strtokIndex = strtok(NULL, marker);
        char data[strlen(strtokIndex)];
        strcpy(data, strtokIndex);

        // Pack received data into packet
        Packet packet;
        packet.id = id;
        packet.type = type;
        packet.data = data;

        // Call pointer callback function
        callback(packet);

        // Received complete buffer
        return true;
    }

    // Didn't receive complete buffer
    return false;
};

bool SimpleSerialProtocol::sendPacket(char* id, char* type, char* data) {
    // Create new packet buffer
    char serialPacketBuffer[serialConnection.getBufferSize()];

    // Pack data in buffer
    sprintf(serialPacketBuffer, "%c%s%c%s%c%s%c", 
        serialConnection.packetStartMarker, 
        id, 
        serialConnection.packetMidMarker,
        type,
        serialConnection.packetMidMarker,
        data, 
        serialConnection.packetEndMarker
    );
    
    // Send buffer to serial device
    serialConnection.send(serialPacketBuffer);

    return true;
};